using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Data.Common;
using System.Data.OleDb;
using System.Data.SqlClient;
using System.Data.SqlTypes;
using System.Xml;
using System.Reflection;
using System.Web;
using System.Web.SessionState;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using System.IO;
using AquaforestTIFFServerStream;
using System.Drawing.Drawing2D;
using System.Drawing.Imaging;
using System.Runtime.InteropServices;

namespace AquaforestTIFFServerStream
{

	//	This class has methods which can be used to customize TIFF Server so that 
	//  for example, the product can read encrypted TIFF images. See section 14.6 of the ref. guide.

	public class AquaforestTIFFServerStream
	{

		// This method returns a TIFF file as a memory MemoryStream.  
		// The supplied implementation returns the TIFF file specified by the TIFFFilePath 
		// parameter as a MemoryStream.

		static public MemoryStream GetStream(string TIFFFilePath)
		{
			SampleCrypto decrypter = new SampleCrypto();
			MemoryStream memoryStream = decrypter.DecryptFile(TIFFFilePath, "");
			return (memoryStream);
		}


		// This method returns the contents of an annotation file as a file as a string.  
		// The supplied implementation returns the annotation file specified by the 
		// AnFilePath parameter as a string.
		static public string GetAnnotations(string AnFilePath)
		{
			if(AnFilePath==null) 
				return("<nots_annot></nots_annot>");

			FileInfo fileInfo=new FileInfo(AnFilePath);
			if(fileInfo.Exists)
			{
				SampleCrypto decrypter = new SampleCrypto();
				MemoryStream memoryStream = decrypter.DecryptFile(AnFilePath, "");
				System.Text.ASCIIEncoding enc = new System.Text.ASCIIEncoding();
				string anString  = enc.GetString(memoryStream.GetBuffer());
				memoryStream.Close();
				return (anString);
			}
			else
			{
				return("<ts_annot></ts_annot>");
			}
		}

		// This method saves the contents of an annotation file.  
		// The supplied implementation saves the string in the specified annotation file.
		static public bool SaveAnnotations(string FilePath, string Annotations)
		{
			try
			{
				writeToTSLog(Annotations+":-:"+FilePath);
				if(Annotations=="")
				{
					FileInfo fileInfo=new FileInfo(FilePath);
					if(fileInfo.Exists)
						fileInfo.Delete();
					return true;
				}
				SampleCrypto encrypter = new SampleCrypto();
				FileStream file= new FileStream(FilePath, FileMode.Create, FileAccess.Write);
				System.Text.Encoding enc = System.Text.Encoding.GetEncoding("windows-1252"); 
				StreamWriter sw = new StreamWriter(file,enc);

				Annotations  = enc.GetString(encrypter.EncryptAnnot(Annotations));
				sw.Write(Annotations);
				sw.Close();
				file.Close();
				return true;
			}
			catch(Exception x)
			{
				writeToTSLog(x.ToString());
				HttpContext.Current.Response.Write(x.ToString());
				HttpContext.Current.Response.End();
				return false;
			}
			
		}	

		public static bool SetZonedImage(string sFilePath, MemoryStream msImageSection)
		{
			Bitmap b=new Bitmap(msImageSection);
			b.Save("c:/temp/y.tif",System.Drawing.Imaging.ImageFormat.Tiff);
			return true;
		}

		public static  void writeToTSLog(string s)
		{
			string TSLogFilePath="d:/yourlogfile.txt";
			
			try
			{
				System.IO.StreamWriter TSLogFile = new System.IO.StreamWriter(TSLogFilePath,true);
				if(TSLogFile!=null)
				{
					TSLogFile.WriteLine(DateTime.Now+":"+HttpContext.Current.Request.RawUrl);
					TSLogFile.WriteLine(s);
					TSLogFile.WriteLine();
					TSLogFile.Flush();
					TSLogFile.Close();
				}
			}
			catch(Exception)
			{
			}
		}	
		static public string ContentType(string ext)
		{
			string contentType;
			switch (ext.ToLower()) 
			{
				case ".doc" : contentType="application/msword";break;
				case ".pdf" : contentType="application/pdf";break;	
				case ".ppt" : contentType="application/ms-powerpoint";break;	
				case ".xls" : contentType="application/vnd.ms-excel";break;			
				default:  contentType="text/html";break;
			}
				
			return(contentType);
		}

		// This method returns true if it is OK to edit the given file
		static public bool Lock(string TIFFFileName)
		{
            return true;

		}
		static public void UnLock(string TIFFFileName)
		{

			
		}

		// This method saves the TIFF and Annotation Streams, using the TIFF File name as a basis
		static public void SaveEditedImages(string TIFFFileName ,MemoryStream[] TIFFStreams,
			string AnnotationFileName, string[] AnnotationStrings)
		{

			int docs=TIFFStreams.Length;
			int i;
			FileStream  fs; 
			BinaryWriter bw; 
			TextWriter tw;
			for(i=0; i<docs; i++)
			{
				if(i==0)
					fs = File.Create(TIFFFileName); 
				else
					fs = File.Create(TIFFFileName.Substring(0,TIFFFileName.LastIndexOf("."))+"_"+i+".tif");
				bw = new BinaryWriter(fs); 
				int sl=(int)TIFFStreams[i].Length;
				byte[] buff=TIFFStreams[i].GetBuffer();
				byte five = (byte) '5';
				for (int iCount = 0; iCount < buff.Length; iCount++)
				{
					buff[iCount] = (byte) (buff[iCount]- five);
				}
				bw.Write(buff,0,sl);
				bw.Close();
				fs.Close();

				
					if(i==0)
						SaveAnnotations(AnnotationFileName,AnnotationStrings[i]);

					else

					SaveAnnotations(AnnotationFileName.Substring(0,AnnotationFileName.LastIndexOf("_"))
							+"_"+i+"_ann.xml",AnnotationStrings[i]);

				
			}
		}


	}

          
	public class SampleCrypto
	{

		public byte [] EncryptAnnot(string anString)
		{
			byte[] buffer = new byte[anString.Length]; 
			System.Text.ASCIIEncoding  encoding=new System.Text.ASCIIEncoding();
			buffer=encoding.GetBytes(anString);
			byte five = (byte) '5';
			for (int iCount = 0; iCount < buffer.Length; iCount++)
			{
				buffer[iCount] = (byte) (buffer[iCount] - five);
			}
			return(buffer);
		}

		public MemoryStream DecryptFile(string sFileIn, string sPassword)
		{
			sFileIn=sFileIn+".enc";
			try
			{
				// First we are going to open the file streams 
				FileStream fsIn = new FileStream(sFileIn, FileMode.Open, FileAccess.Read); 
				MemoryStream msOut = new MemoryStream();

				// Now will will initialize a buffer and will be 
				// processing the input file in chunks. 

				int bufferLen = 4096; 
				byte[] buffer = new byte[bufferLen]; 
				int bytesRead; 
				byte five = (byte) '5';

				do 
				{ 
					bytesRead = fsIn.Read(buffer, 0, bufferLen); 
					for (int iCount = 0; iCount < bytesRead; iCount++)
					{
						if(sPassword=="") buffer[iCount] = (byte) (buffer[iCount] + five);
					}
					msOut.Write(buffer, 0, bytesRead);
					msOut.Flush();
				} while(bytesRead != 0); 

				//msOut.Close();
				fsIn.Close();
				return msOut;
			}

			catch(Exception ex)
			{
				throw ex;
			}

		}

	}


}

